// ******************************************************************************
// @file    boot_stage_2.c
// @author  Milandr Application Team
// @version V0.1.0
// @date    12/02/2024
// @brief   Main program body of bootloader for MCU's with CHIP_ID = 215 or 217.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"

#include "boot_stage_2.h"
#include "boot_stage_3_flash.h"
#include "boot_stage_3_uart.h"
#include "boot_common.h"
#include "system_MDR32VF0xI_it.h"

#include <stdbool.h>

// ------------------------------------------------------------------------------
// Main function
int main (void)
{
    uint32_t boot_mode;
    uint32_t tmp;

    // Core clock after reset is HSE/16 (512kHz). Peripheral clock is HSI (8MHz).
    // RST clock enabled by POR and never disable by this software.
    // Enable clock for BKP and disable it after FPOR set in low power modes.

    rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_BKP);

    // Init CLIC
     CLIC_SetTrapVector(CLIC_PRIVILEGE_MODE_IRQ_M, Trap_IRQHandler);

    // Check FPOR bit in BKP (first run after POR) and choose
    // boot mode from IO pins (1-st run) or BKP mode field (2-nd and more run).

    if ((MDR_BKP->LDO & BKP_LDO_FPOR_Msk) == BKP_LDO_FPOR)
    {
        boot_mode = (MDR_BKP->LDO & BKP_LDO_MODE_Msk) >> BKP_LDO_MODE_Pos;

        // Trying to setup manufacture trimmings with strategy:
        // - If checksum over manufacture info fields is NOT valid, then setup Manufacture defaults.
        // - Skip setup TRIM_LDO and TRIM_TS, because it's not present by Manufacture.

        setup_manufacture_trimings ();
    }

    else
    {
        // Init mode port and make >1ms delay for discharge mode IO over pulldowns.
        // In this delay setup manufacture trimmings.
        // Also in this delay try to close errata issue #0002:
        // set an clear BLDO boost mode if main Ucc upper minimum PVD level.

        rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_POWER | RST_CLK_PER2_CLOCK_PCLK_EN_PORTC);

        init_mode_port (MDR_PORTC);

        setup_manufacture_trimings ();

        // Setup PVD with minimum level PLS == 0 and clear results (2-x write).

        MDR_POWER->PVDCS &= ~(POWER_PVDCS_PLS_Msk);
        MDR_POWER->PVDCS |= (POWER_PVDCS_PVDEN);

        MDR_POWER->PVDCS &= ~(POWER_PVDCS_PVD);
        MDR_POWER->PVDCS &= ~(POWER_PVDCS_PVD);

        // Get PVD flag, turn off PVD and deinit clocks.

        tmp = (MDR_POWER->PVDCS & POWER_PVDCS_PVD);

        MDR_POWER->PVDCS &= ~(POWER_PVDCS_PVDEN); // TODO: PVD_EN set by software, but not clear by software, only by POR.

        rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_POWER);

        if (tmp == POWER_PVDCS_PVD)
        {
            MDR_BKP->WPR = 0x8555AAA1;
            tmp = MDR_BKP->LDO;
            tmp &= ~(BKP_LDO_SW_OFF_Msk | BKP_LDO_COVDET_Msk);
            tmp |= BKP_LDO_BLDO_BOOST;
            MDR_BKP->LDO = tmp;

            delay_50us_core_500kHz();

            tmp &= ~BKP_LDO_BLDO_BOOST;
            MDR_BKP->LDO = tmp;
            MDR_BKP->WPR = 0;
        }

        else
        {
            delay_50us_core_500kHz();
        }

        // Get boot mode from IO, deinit IO and deinit clocks.

        boot_mode = (MDR_PORTC->RXTX & (PORT_RXTX_RXTX1 | PORT_RXTX_RXTX0));

        deinit_mode_port (MDR_PORTC);

        rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_PORTC);

        bkp_write_mode (boot_mode);
    }

    switch (boot_mode)
    {
        case mode_flash: // Finally run App from flash. Low power mode.
        {
            bkp_set_fpor ();
            rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_BKP);

            run_app (FLASH_BASE);
            break;
        }

        case mode_ram: // Finally run App from RAM. Low power mode.
        {
            bkp_set_fpor ();
            rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_BKP);

            run_app (RAM_TCMA_BASE);
            break;
        }

        case mode_uart_0: // Finally setup UART, pick up baud rate and be ready to receive commands from host.
        case mode_uart_1:
        {
        	// Change core clock from HSI (8 MHz) because this modes is no low power.

        	tmp = MDR_RST_CLK->CPU_CLOCK;
        	tmp &= ~(RST_CLK_CPU_CLOCK_HCLK_SEL_Msk);
        	tmp |=  (RST_CLK_CPU_CLOCK_HCLK_SEL_HSI);
        	MDR_RST_CLK->CPU_CLOCK = tmp;

        	// Also on clk on Flash controller for for compatibility with boot rev 0.
        	// Active clocks: RST, BKP, FLASH.
        	// Uart clock and IO setups inside run_app_uart.

        	rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_FLASH);

            run_app_uart ();
            break;       // unreachable code.
        }

        default:
        {
            while (true); // In other cases choose a strategy: deadlock.
            break;
        }
    }

    while (true);   // Deadlock.
    return (0);		// unreachable code.
}
// ------------------------------------------------------------------------------
