// ******************************************************************************
// @file    boot_stage_3_uart.c
// @author  Milandr Application Team
// @version V0.1.0
// @date    12/02/2024
// @brief   Functions for work with UART.
// ******************************************************************************
// THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
// WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
// THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
// CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.

// Includes
#include "system_MDR32VF0xI_config.h"
#include "boot_stage_3_uart.h"
#include "boot_common.h"

// ------------------------------------------------------------------------------
void run_app_uart (void)
{
    uint32_t divisor;
    uint32_t div_lo;
    uint32_t err;
    uint32_t sym;
    uint32_t i;
    uint32_t tmp;

    // Init clocks for UART1 and PORTB
    rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_UART1 | RST_CLK_PER2_CLOCK_PCLK_EN_PORTB);

    MDR_RST_CLK->UART_CLOCK |= RST_CLK_UART_CLOCK_UART1_CLK_EN;

    uart_io_init (MDR_PORTB);

    rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_PORTB);

    do
    {
        div_lo = divisor = DIV_MIN;
        for (; (divisor < DIV_MAX) && (divisor - div_lo < DIV_DLT * WO_ERR_CNT);)
        {
            uart_set_baud (divisor);
            uart_receive_byte ();
            divisor += DIV_DLT;
            if (uart_sync_err ())
            {
                div_lo = divisor;
            }
        }
    }
    while (divisor - div_lo < DIV_DLT * WO_ERR_CNT);

    for (; divisor < DIV_MAX; divisor += DIV_DLT)
    {
        uart_set_baud (divisor);
        uart_receive_byte ();
        if (uart_sync_err ())
        {
            break;
        }
    }

    divisor = (div_lo + divisor) >> 1;
    uart_set_baud (divisor);
    for (i = 0; i < 16; i++)
    {
        uart_receive_byte ();
    }
    sym = CMD_CR;

    for (;;)
    {
        err = 0;
        if (sym >> 8)
        {
            err = ERR_CHN;
        }

        if (!err)
        {
            switch (sym)
            {
                case CMD_SYNC:
                {
                    break;
                }

                case CMD_CR:
                {
                    uart_send_byte (CMD_CR);
                    uart_send_byte (0x0A);
                    uart_send_byte ('>');
                    break;
                }

                case CMD_BAUD:
                {
                    tmp = uart_receive_param ();
                    if (tmp == ~0UL)
                    {
                        err = ERR_CHN;
                        break;
                    }

                    if (!tmp)
                    {
                        err = ERR_BAUD;
                        break;
                    }

                    tmp = ((uint32_t) (divisor * BAUD_RATE)) / tmp;

                    if ((tmp < 64) || (tmp >= (1 << 22)))
                    {
                        err = ERR_BAUD;
                        break;
                    }

                    uart_send_byte (CMD_BAUD);
                    uart_set_baud (tmp);
                    break;
                }

                case CMD_LOAD:
                case CMD_VFY:
                {
                    uint32_t addr = 0;
                    uint32_t size = 0;
                    uint32_t word = 0;

                    addr = uart_receive_param ();
                    size = uart_receive_param ();
                    if ((addr == ~0UL) || (size == ~0UL))
                    {
                        err = ERR_CHN;
                        break;
                    }
                    uart_send_byte (sym);

                    size += addr;
                    for (; addr < size; addr++)
                    {
                        if (sym == CMD_VFY)
                        {
                            // workaround for peripheral registers
                            if ((addr & 0xF0000000) == PERIPH_BASE)
                            {
                                uart_send_byte (*(volatile uint32_t*) (addr - addr % 4) >> (8 * (addr % 4)));
                            }
                            else
                            {
                                uart_send_byte (*(volatile uint8_t*) addr);
                            }
                        }
                        else
                        {
                            tmp = uart_receive_byte ();
                            if (tmp >> 8)
                            {
                                err = ERR_CHN;
                                break;
                            }

                            // workaround for peripheral registers
                            if ((addr & 0xF0000000) == PERIPH_BASE)
                            {
                                word = (tmp << 24) | (word >> 8);

                                if ((addr % 4) == 3)
                                {
                                    *(volatile uint32_t*) (addr - addr % 4) = word;
                                }
                            }
                            else
                            {
                                *(volatile uint8_t*) addr = (uint8_t) tmp;
                            }
                        }
                    }

                    if (err)
                    {
                        break;
                    }
                    uart_send_byte (REPLY_OK);
                    break;
                }

                case CMD_RUN:
                {
                    tmp = uart_receive_param ();
                    if (tmp == ~0UL)
                    {
                        err = ERR_CHN;
                        break;
                    }
                    uart_send_byte (CMD_RUN);
                    uart_wait_busy ();

                    // Finally set FPOR, disable BKP clock and run User App.
                    bkp_set_fpor ();
                    run_app (tmp);
                    break;          // User App may return here
                }

                default:
                {
                    err = ERR_CMD;
                }
            }
        }

        if (err)
        {
            uart_send_byte (REPLY_ERR);
            uart_send_byte (err);
        }

        sym = uart_receive_byte ();
    }
}
// ------------------------------------------------------------------------------
// @formatter:off
void uart_io_init (MDR_PORT_TypeDef *PORT)
{
    uint32_t chip_id;

    rst_clk_per2_set (RST_CLK_PER2_CLOCK_PCLK_EN_FLASH);
    chip_id = (MDR_FLASH->CHIP_ID_CTRL & FLASH_CHIP_ID_CTRL_CHIP_ID_Msk); // ID << 2!
    rst_clk_per2_clear (RST_CLK_PER2_CLOCK_PCLK_EN_FLASH);

    PORT->OE       &= ~(PORT_OE_OE0 | PORT_OE_OE1);                            // Clear output enable, setup as input UART_RX / UART_TX
    PORT->FUNC     &= ~(PORT_FUNC_MODE1_Msk | PORT_FUNC_MODE0_Msk);            // Clear func for UART_RX / UART_TX
    PORT->PULL     &= ~(PORT_PULL_PULL_UP1 | PORT_PULL_PULL_UP0 |
                           PORT_PULL_PULL_DOWN1 | PORT_PULL_PULL_DOWN0);       // Disconnect pullup / pulldown for UART_RX / UART_TX

    PORT->FUNC     |=  (PORT_FUNC_MODE1_MAIN | PORT_FUNC_MODE0_MAIN);          // Set func MAIN for UART_RX / UART_TX
    PORT->ANALOG   |=  (PORT_ANALOG_ANALOG_EN1 | PORT_ANALOG_ANALOG_EN0);      // Set digital mode for UART_RX / UART_TX

    switch (chip_id)
    {
        case FLASH_CHIP_ID_CTRL_CHIP_ID215:
        {
            PORT->PWR &= ~(PORT_PWR_PWR0);                                          // Set max_power for UART_TX
            break;
        }

        case FLASH_CHIP_ID_CTRL_CHIP_ID217:
        {
            PORT->PWR |= (PORT_PWR_PWR0);                                          // Set max_power for UART_TX
            break;
        }

        default: break;
    }

    PORT->OE       |=  PORT_OE_OE0;                                            // Set output for UART_TX
}
// @formatter:on

// ------------------------------------------------------------------------------
void uart_set_baud (uint32_t divider)
{
    while (MDR_UART1->FR & UART_FR_BUSY);

    MDR_UART1->CR = 0;
    MDR_UART1->IBRD = divider >> 6;
    MDR_UART1->FBRD = divider & 0x3F;
    MDR_UART1->LCR_H = (3 << UART_LCR_H_WLEN10_Pos);
    MDR_UART1->CR = (UART_CR_UARTEN | UART_CR_TXE | UART_CR_RXE);
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void uart_send_byte (uint32_t ch)
{
    uint32_t tmp;

    //Wait for the previous byte
    while ((tmp = MDR_UART1->FR) & UART_FR_TXFF);

    //flush receive buffer as we are not in receiving state
    if (tmp & UART_FR_RXFF)
    {
        tmp = MDR_UART1->DR;
    }

    MDR_UART1->DR = ch;
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
void uart_wait_busy (void)
{
    // Wait until all characters have been transferred to the line, including stop bits.
    while (MDR_UART1->FR & UART_FR_BUSY);
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
uint32_t uart_receive_byte (void)
{
    while (MDR_UART1->FR & UART_FR_RXFE);
    return (MDR_UART1->DR & (UART_DR_BE_Msk | UART_DR_PE_Msk | UART_DR_FE_Msk | UART_DR_DATA_Msk));
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
uint32_t uart_sync_err (void)
{
    uint32_t i;
    for (i = 0; i < 16; i++)
    {
        while ((MDR_UART1->FR & UART_FR_RXFE));
        if ((MDR_UART1->DR & (UART_DR_BE_Msk | UART_DR_PE_Msk | UART_DR_FE_Msk | UART_DR_DATA_Msk)) != CMD_SYNC)
        {
            return (1);
        }
    }
    return (0);
}
// ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------
uint32_t uart_receive_param (void)
{
    uint32_t tmp;
    uint32_t res = 0;

    for (tmp = 0; tmp < 0x4000; tmp += 0x1000)
    {
        while (MDR_UART1->FR & UART_FR_RXFE);
        tmp |= MDR_UART1->DR & (UART_DR_BE_Msk | UART_DR_PE_Msk | UART_DR_FE_Msk | UART_DR_DATA_Msk);
        res = (res >> 8) | (tmp << 24); // TODO: Why need (res >> 8) ??
        tmp &= ~0xFFUL;
    }
    return ((tmp & 0xF00) ? ~0UL : res);
}
// ------------------------------------------------------------------------------
