/**
 *******************************************************************************
 * @file    link_BOOT.ld
 * @author  Milandr Application Team
 * @version V1.0.0
 * @date    09/11/2023
 * @brief   Linker script file for MDR1206.
 *******************************************************************************
 * <br><br>
 *
 * THE PRESENT FIRMWARE IS FOR GUIDANCE ONLY. IT AIMS AT PROVIDING CUSTOMERS
 * WITH CODING INFORMATION REGARDING MILANDR'S PRODUCTS IN ORDER TO FACILITATE
 * THE USE AND SAVE TIME. MILANDR SHALL NOT BE HELD LIABLE FOR ANY
 * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES RESULTING
 * FROM THE CONTENT OF SUCH FIRMWARE AND/OR A USE MADE BY CUSTOMERS OF THE
 * CODING INFORMATION CONTAINED HEREIN IN THEIR PRODUCTS.
 *
 * <h2><center>&copy; COPYRIGHT {YYYY} Milandr</center></h2>
 *******************************************************************************
 */

/* It defines following symbols, which code can use without definition:
 *   __text_start
 *   __text_end
 *   __data_start
 *   __data_end
 *   __end
 *   __stack_top
 *   __stack_size
 *   __stack_limit
 *   __heap_top
 *   __heap_min_size
 *   __heap_size
 *   __heap_limit
 */

OUTPUT_ARCH( "riscv" )
ENTRY(_start)

/* __stack_size - total stack size */
__stack_size    = 128;
/* __heap_min_size - minimum heap size, used to check if space can be allocated. The total heap size (__heap_size) takes up all space from end of the .bss to beginning of .stack */
__heap_min_size = 4;

/* Specify the memory areas */
MEMORY
{
    BOOT_CODE (rx)      : ORIGIN = 0x00020000, LENGTH = 15K
    BOOT_RAM  (xrw)     : ORIGIN = 0x40010000 + 32K - 256, LENGTH = 256
}


/* Define output sections */
/* ">vma_region" - VMA (virtual memory address),
   "AT>lma_region" - LMA (load memory address) */

SECTIONS
{
    /* The startup code goes first into BOOT_CODE */
	/* The program code and other data goes into BOOT_CODE */
	
    .text :
    {  
		. = ALIGN(4);
		__text_start = .;
        *(.text.init)
		*(.text*)
		*(.rodata*)
        *(.srodata*)
        . = ALIGN(4);
    } >BOOT_CODE
	__data_load_start = .;

    /* Initialized data sections goes into BOOT_RAM from BOOT_CODE*/
	/* Read-only data goes into BOOT_RAM from BOOT_CODE */
    .data : AT (__data_load_start)
    {
		. = ALIGN(4);
        __data_start = .;
        *(.data*)
        *(.sdata*)
		. = ALIGN(4);
		__data_end = .;
    } >BOOT_RAM
	__end = .;

    
      /* Stack and Heap symbol definitions */
      __stack_top = ORIGIN(BOOT_RAM) + LENGTH(BOOT_RAM);
      __stack_limit = __stack_top - __stack_size;
      __heap_top = __end;
      __heap_size = __stack_limit - __heap_top;
      __heap_limit = __heap_top + __heap_size;

    /* Check allocation for heap of size __heap_min_size */
    ASSERT(__heap_size >= __heap_min_size, "Unable to allocate __heap_min_size for heap!")

    /* .heap section doesn't contains any symbols. 
     It is only used for linker to calculate size of heap sections, and assign values to heap symbols */
    .heap __heap_top :
    {
        . += __heap_size;
    } >BOOT_RAM

    /* .stack section doesn't contains any symbols. 
     It is only used for linker to calculate size of stack sections, and assign values to stack symbols */
    .stack __stack_limit :
    {
        . += __stack_size;
    } >BOOT_RAM
    
    /* Check for alignment of BOOT RAM by 32-bytes boundary  */
    /* ASSERT( (LENGTH(BOOT_RAM) % 32) == 0 , "Please, select size of RAM aligned to 32-bytes boudary. It needed for fast RAM cleaning. See startup code.")*/
	
	/* Check data section is not zero */
	ASSERT ((__data_end - __data_start == 0), "Now data section is not zero. Please, enable 'Copy data to RAM' loop in startup code and fix this assert to '>'")
	
}